#***************************************************************************
#*   Copyright (c) 2011 Yorik van Havre <yorik@uncreated.net>              *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU Lesser General Public License (LGPL)    *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Library General Public License for more details.                  *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with this program; if not, write to the Free Software   *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************

__title__  = "FreeCAD Arch API"
__author__ = "Yorik van Havre"
__url__    = "https://www.freecad.org"

## \defgroup ARCH Arch
#  \ingroup PYTHONWORKBENCHES
#  \brief Architecture and BIM tools
#
#  This module provides tools specialized in Building Information Modeling (BIM).
#  such as convenience tools to build walls, windows or structures, and
#  IFC import/export capabilities.

'''The Arch module provides tools specialized in BIM modeling.'''

import FreeCAD
if FreeCAD.GuiUp:
    import FreeCADGui
    FreeCADGui.updateLocale()
QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate


# generic functions

from ArchCommands import *
from ArchWindowPresets import *


# TODO: migrate this one

from ArchStructure import *
from ArchSpace import *

# make functions

def makeAxis(num=1,size=1000,name=None):

    '''makeAxis([num],[size],[name]): makes an Axis set
    based on the given number of axes and interval distances'''

    import ArchAxis
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Axis")
    obj.Label = name if name else translate("Arch","Axes")
    ArchAxis._Axis(obj)
    if FreeCAD.GuiUp:
        ArchAxis._ViewProviderAxis(obj.ViewObject)
    if num:
        dist = []
        angles = []
        for i in range(num):
            if i == 0:
                dist.append(0)
            else:
                dist.append(float(size))
            angles.append(float(0))
        obj.Distances = dist
        obj.Angles = angles
    FreeCAD.ActiveDocument.recompute()
    return obj


def makeAxisSystem(axes,name=None):

    '''makeAxisSystem(axes,[name]): makes a system from the given list of axes'''

    import ArchAxisSystem
    if not isinstance(axes,list):
        axes = [axes]
    obj = FreeCAD.ActiveDocument.addObject("App::FeaturePython","AxisSystem")
    obj.Label = name if name else translate("Arch","Axis System")
    ArchAxisSystem._AxisSystem(obj)
    obj.Axes = axes
    if FreeCAD.GuiUp:
        ArchAxisSystem._ViewProviderAxisSystem(obj.ViewObject)
    FreeCAD.ActiveDocument.recompute()
    return obj


def makeBuildingPart(objectslist=None,baseobj=None,name=None):

    '''makeBuildingPart([objectslist],[name]): creates a buildingPart including the
    objects from the given list.'''

    import ArchBuildingPart
    obj = FreeCAD.ActiveDocument.addObject("App::GeometryPython","BuildingPart")
    #obj = FreeCAD.ActiveDocument.addObject("App::FeaturePython","BuildingPart")
    obj.Label = name if name else translate("Arch","BuildingPart")
    ArchBuildingPart.BuildingPart(obj)
    obj.IfcType = "Building Element Part"
    if FreeCAD.GuiUp:
        ArchBuildingPart.ViewProviderBuildingPart(obj.ViewObject)
    if objectslist:
        obj.addObjects(objectslist)
    return obj


def makeFloor(objectslist=None,baseobj=None,name=None):

    """makes a BuildingPart and turns it into a Floor/Level"""

    obj = makeBuildingPart(objectslist)
    obj.Label = name if name else translate("Arch","Level")
    obj.IfcType = "Building Storey"
    return obj


def makeBuilding(objectslist=None,baseobj=None,name=None):

    """makes a BuildingPart and turns it into a Building"""

    import ArchBuildingPart
    obj = makeBuildingPart(objectslist)
    obj.Label = name if name else translate("Arch","Building")
    obj.IfcType = "Building"
    t = QT_TRANSLATE_NOOP("App::Property","The type of this building")
    obj.addProperty("App::PropertyEnumeration","BuildingType","Building",t)
    obj.BuildingType = ArchBuildingPart.BuildingTypes
    if FreeCAD.GuiUp:
        obj.ViewObject.ShowLevel = False
        obj.ViewObject.ShowLabel = False
    return obj


def convertFloors(floor=None):

    """convert the given Floor or Building (or all Arch Floors from the
    active document if none is given) into BuildingParts"""

    import Draft
    import ArchBuildingPart
    todel = []
    if floor:
        objset = [floor]
    else:
        objset = FreeCAD.ActiveDocument.Objects
    for obj in objset:
        if Draft.getType(obj) in ["Floor","Building"]:
            nobj = makeBuildingPart(obj.Group)
            if Draft.getType(obj) == "Floor":
                nobj.IfcType = "Building Storey"
            else:
                nobj.IfcType = "Building"
                t = QT_TRANSLATE_NOOP("App::Property","The type of this building")
                nobj.addProperty("App::PropertyEnumeration","BuildingType","Building",t)
                nobj.BuildingType = ArchBuildingPart.BuildingTypes
            label = obj.Label
            for parent in obj.InList:
                if hasattr(parent,"Group"):
                    if obj in parent.Group:
                        parent.addObject(nobj)
                        #g = parent.Group
                        #g.append(nobj)
                        #parent.Group = g
            todel.append(obj.Name)
            if obj.ViewObject:
                # some bug makes this trigger even efter the object has been deleted...
                obj.ViewObject.Proxy.Object = None
                # in case FreeCAD doesn't allow 2 objs with same label
            obj.Label = obj.Label+" to delete"
            nobj.Label = label
    for n in todel:
        from DraftGui import todo
        todo.delay(FreeCAD.ActiveDocument.removeObject,n)


def makeCurtainWall(baseobj=None,name=None):

    """makeCurtainWall([baseobj],[name]): Creates a curtain wall in the active document"""

    import ArchCurtainWall
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","CurtainWall")
    obj.Label = name if name else translate("Arch","Curtain Wall")
    ArchCurtainWall.CurtainWall(obj)
    if FreeCAD.GuiUp:
        ArchCurtainWall.ViewProviderCurtainWall(obj.ViewObject)
    if baseobj:
        obj.Base = baseobj
        if FreeCAD.GuiUp:
            baseobj.ViewObject.hide()
    return obj


def makeEquipment(baseobj=None,placement=None,name=None):

    """makeEquipment([baseobj],[placement],[name]): creates an equipment object
    from the given base object."""

    import ArchEquipment
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Equipment")
    obj.Label = name if name else translate("Arch","Equipment")
    ArchEquipment._Equipment(obj)
    if baseobj:
        if baseobj.isDerivedFrom("Mesh::Feature"):
            obj.Mesh = baseobj
        else:
            obj.Base = baseobj
    if placement:
        obj.Placement = placement
    if FreeCAD.GuiUp:
        ArchEquipment._ViewProviderEquipment(obj.ViewObject)
        if baseobj:
            baseobj.ViewObject.hide()
    return obj


def makeFence(section, post, path):

    """Makes a Fence object"""

    import ArchFence
    obj = FreeCAD.ActiveDocument.addObject('Part::FeaturePython', 'Fence')
    ArchFence._Fence(obj)
    obj.Section = section
    obj.Post = post
    obj.Path = path
    if FreeCAD.GuiUp:
        ArchFence._ViewProviderFence(obj.ViewObject)
        ArchFence.hide(section)
        ArchFence.hide(post)
        ArchFence.hide(path)
    return obj


def makeFrame(baseobj,profile,name=None):

    """makeFrame(baseobj,profile,[name]): creates a frame object from a base sketch (or any other object
    containing wires) and a profile object (an extrudable 2D object containing faces or closed wires)"""

    import ArchFrame
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Frame")
    obj.Label = name if name else translate("Arch","Frame")
    ArchFrame._Frame(obj)
    if FreeCAD.GuiUp:
        ArchFrame._ViewProviderFrame(obj.ViewObject)
    if baseobj:
        obj.Base = baseobj
    if profile:
        obj.Profile = profile
        if FreeCAD.GuiUp:
            profile.ViewObject.hide()
    return obj


def makeGrid(name=None):

    '''makeGrid([name]): makes a grid object'''

    import ArchGrid
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Grid")
    obj.Label = name if name else translate("Arch","Grid")
    ArchGrid.ArchGrid(obj)
    if FreeCAD.GuiUp:
        ArchGrid.ViewProviderArchGrid(obj.ViewObject)
        obj.ViewObject.Transparency = 85
    FreeCAD.ActiveDocument.recompute()
    return obj


def makeMaterial(name=None,color=None,transparency=None):

    '''makeMaterial([name],[color],[transparency]): makes an Material object'''

    import ArchMaterial
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("App::MaterialObjectPython","Material")
    obj.Label = name if name else translate("Arch","Material")
    ArchMaterial._ArchMaterial(obj)
    if FreeCAD.GuiUp:
        ArchMaterial._ViewProviderArchMaterial(obj.ViewObject)
    getMaterialContainer().addObject(obj)
    if color:
        obj.Color = color[:3]
        if len(color) > 3:
            obj.Transparency = color[3]*100
    if transparency:
        obj.Transparency = transparency
    return obj


def makeMultiMaterial(name=None):

    '''makeMultiMaterial([name]): makes an MultiMaterial object'''

    import ArchMaterial
    obj = FreeCAD.ActiveDocument.addObject("App::FeaturePython","MultiMaterial")
    obj.Label = name if name else translate("Arch","MultiMaterial")
    ArchMaterial._ArchMultiMaterial(obj)
    if FreeCAD.GuiUp:
        ArchMaterial._ViewProviderArchMultiMaterial(obj.ViewObject)
    getMaterialContainer().addObject(obj)
    return obj


def getMaterialContainer():

    '''getMaterialContainer(): returns a group object to put materials in'''

    import ArchMaterial
    for obj in FreeCAD.ActiveDocument.Objects:
        if obj.Name == "MaterialContainer":
            return obj
    obj = FreeCAD.ActiveDocument.addObject("App::DocumentObjectGroupPython","MaterialContainer")
    obj.Label = "Materials"
    ArchMaterial._ArchMaterialContainer(obj)
    if FreeCAD.GuiUp:
        ArchMaterial._ViewProviderArchMaterialContainer(obj.ViewObject)
    return obj


def getDocumentMaterials():

    '''getDocumentMaterials(): returns all the arch materials of the document'''

    for obj in FreeCAD.ActiveDocument.Objects:
        if obj.Name == "MaterialContainer":
            mats = []
            for o in obj.Group:
                if o.isDerivedFrom("App::MaterialObjectPython"):
                    mats.append(o)
            return mats
    return []


def makePanel(baseobj=None,length=0,width=0,thickness=0,placement=None,name=None):

    '''makePanel([baseobj],[length],[width],[thickness],[placement],[name]): creates a
    panel element based on the given profile object and the given
    extrusion thickness. If no base object is given, you can also specify
    length and width for a simple cubic object.'''

    import ArchPanel
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Panel")
    obj.Label = name if name else translate("Arch","Panel")
    ArchPanel._Panel(obj)
    if FreeCAD.GuiUp:
        ArchPanel._ViewProviderPanel(obj.ViewObject)
    if baseobj:
        obj.Base = baseobj
        if FreeCAD.GuiUp:
            obj.Base.ViewObject.hide()
    if width:
        obj.Width = width
    if thickness:
        obj.Thickness = thickness
    if length:
        obj.Length = length
    return obj


def makePanelCut(panel,name=None):

    """makePanelCut(panel,[name]) : Creates a 2D view of the given panel
    in the 3D space, positioned at the origin."""

    import ArchPanel
    view = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","PanelCut")
    view.Label = name if name else translate("Arch","View of")+" "+panel.Label
    ArchPanel.PanelCut(view)
    view.Source = panel
    if FreeCAD.GuiUp:
        ArchPanel.ViewProviderPanelCut(view.ViewObject)
    return view


def makePanelSheet(panels=[],name=None):

    """makePanelSheet([panels],[name]) : Creates a sheet with the given panel cuts
    in the 3D space, positioned at the origin."""

    import ArchPanel
    sheet = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","PanelSheet")
    sheet.Label = name if name else translate("Arch","PanelSheet")
    ArchPanel.PanelSheet(sheet)
    if panels:
        sheet.Group = panels
    if FreeCAD.GuiUp:
        ArchPanel.ViewProviderPanelSheet(sheet.ViewObject)
    return sheet


def makePipe(baseobj=None,diameter=0,length=0,placement=None,name=None):

    "makePipe([baseobj],[diameter],[length],[placement],[name]): creates an pipe object from the given base object"

    import ArchPipe
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj= FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Pipe")
    obj.Label = name if name else translate("Arch","Pipe")
    ArchPipe._ArchPipe(obj)
    if FreeCAD.GuiUp:
        ArchPipe._ViewProviderPipe(obj.ViewObject)
        if baseobj:
            baseobj.ViewObject.hide()
    if baseobj:
        obj.Base = baseobj
    else:
        if length:
            obj.Length = length
        else:
            obj.Length = 1000
    if diameter:
        obj.Diameter = diameter
    else:
        obj.Diameter = params.get_param_arch("PipeDiameter")
    obj.Width = obj.Diameter
    obj.Height = obj.Diameter
    if placement:
        obj.Placement = placement
    return obj


def makePipeConnector(pipes,radius=0,name=None):

    "makePipeConnector(pipes,[radius],[name]): creates a connector between the given pipes"

    import ArchPipe
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj= FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Connector")
    obj.Label = name if name else translate("Arch","Connector")
    ArchPipe._ArchPipeConnector(obj)
    obj.Pipes = pipes
    if not radius:
        radius = pipes[0].Diameter
    obj.Radius = radius
    if FreeCAD.GuiUp:
        ArchPipe._ViewProviderPipe(obj.ViewObject)
    return obj


def makeProfile(profile=[0,'REC','REC100x100','R',100,100]):

    '''makeProfile(profile): returns a shape  with the face defined by the profile data'''

    import ArchProfile
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::Part2DObjectPython", "Profile")
    obj.Label = profile[2] + "_"
    if profile[3]=="C":
        ArchProfile._ProfileC(obj, profile)
    elif profile[3]=="H":
        ArchProfile._ProfileH(obj, profile)
    elif profile[3]=="R":
        ArchProfile._ProfileR(obj, profile)
    elif profile[3]=="RH":
        ArchProfile._ProfileRH(obj, profile)
    elif profile[3]=="U":
        ArchProfile._ProfileU(obj, profile)
    elif profile[3]=="L":
        ArchProfile._ProfileL(obj, profile)
    elif profile[3]=="T":
        ArchProfile._ProfileT(obj, profile)
    else :
        print("Profile not supported")
    if FreeCAD.GuiUp:
        ArchProfile.ViewProviderProfile(obj.ViewObject)
    return obj


def makeProject(sites=None, name=None):

    """Create an Arch project.

    If sites are provided, add them as children of the new project.

    Parameters
    ----------
    sites: list of <Part::FeaturePython>, optional
        Sites to add as children of the project. Ultimately this could be
        anything, however.
    name: str, optional
        The label for the project.

    Returns
    -------
    <Part::FeaturePython>
        The created project.

    WARNING: This object is obsoleted in favour of the NativeIFC project
    """

    import ArchProject
    import Part
    if not FreeCAD.ActiveDocument:
        return FreeCAD.Console.PrintError("No active document. Aborting\n")
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython", "Project")
    obj.Label = name if name else translate("Arch", "Project")
    ArchProject._Project(obj)
    if FreeCAD.GuiUp:
        ArchProject._ViewProviderProject(obj.ViewObject)
    if sites:
        obj.Group = sites
    return obj


def makeRebar(baseobj=None,sketch=None,diameter=None,amount=1,offset=None,name=None):

    """makeRebar([baseobj],[sketch],[diameter],[amount],[offset],[name]):
    adds a Reinforcement Bar object to the given structural object,
    using the given sketch as profile."""

    import ArchRebar
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Rebar")
    obj.Label = name if name else translate("Arch","Rebar")
    ArchRebar._Rebar(obj)
    if FreeCAD.GuiUp:
        ArchRebar._ViewProviderRebar(obj.ViewObject)
    if baseobj and sketch:
        if hasattr(sketch,"AttachmentSupport"):
            if sketch.AttachmentSupport:
                if isinstance(sketch.AttachmentSupport,tuple):
                    if sketch.AttachmentSupport[0] == baseobj:
                        sketch.AttachmentSupport = None
                elif sketch.AttachmentSupport == baseobj:
                    sketch.AttachmentSupport = None
        obj.Base = sketch
        if FreeCAD.GuiUp:
            sketch.ViewObject.hide()
        obj.Host = baseobj
    elif sketch and not baseobj:
        # a rebar could be based on a wire without the existence of a Structure
        obj.Base = sketch
        if FreeCAD.GuiUp:
            sketch.ViewObject.hide()
        obj.Host = None
    elif baseobj and not sketch:
        obj.Shape = baseobj.Shape
    if diameter:
        obj.Diameter = diameter
    else:
        obj.Diameter = params.get_param_arch("RebarDiameter")
    obj.Amount = amount
    obj.Document.recompute()
    if offset is not None:
        obj.OffsetStart = offset
        obj.OffsetEnd = offset
    else:
        obj.OffsetStart = params.get_param_arch("RebarOffset")
        obj.OffsetEnd = params.get_param_arch("RebarOffset")
    obj.Mark = obj.Label
    return obj


def makeReference(filepath=None, partname=None, name=None):

    """makeReference([filepath],[partname],[name]): Creates an Arch Reference object"""

    import ArchReference
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","ArchReference")
    obj.Label = name if name else translate("Arch","External Reference")
    ArchReference.ArchReference(obj)
    if FreeCAD.GuiUp:
        ArchReference.ViewProviderArchReference(obj.ViewObject)
    if filepath:
        obj.File = filepath
    if partname:
        obj.Part = partname
    import Draft
    Draft.select(obj)
    return obj


def makeRoof(baseobj=None,
             facenr=0,
             angles=[45.0],
             run=[250.0],
             idrel=[-1],
             thickness=[50.0],
             overhang=[100.0],
             name=None):

    '''makeRoof(baseobj, [facenr], [angle], [name]): Makes a roof based on
    a closed wire or an object.

    You can provide a list of angles, run, idrel, thickness, overhang for
    each edge in the wire to define the roof shape. The default for angle is
    45 and the list is automatically completed to match the number of edges
    in the wire.

    If the base object is a solid the roof uses its shape.
    '''

    import ArchRoof
    import Part
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython", "Roof")
    obj.Label = name if name else translate("Arch", "Roof")
    baseWire = None
    ArchRoof._Roof(obj)
    if FreeCAD.GuiUp:
        ArchRoof._ViewProviderRoof(obj.ViewObject)
    if baseobj:
        obj.Base = baseobj
        if hasattr(obj.Base, "Shape"):
            if obj.Base.Shape.Solids:
                if FreeCAD.GuiUp:
                    obj.Base.ViewObject.hide()
            else:
                if (obj.Base.Shape.Faces and obj.Face):
                    baseWire = obj.Base.Shape.Faces[obj.Face-1].Wires[0]
                    if FreeCAD.GuiUp:
                        obj.Base.ViewObject.hide()
                elif obj.Base.Shape.Wires:
                    baseWire = obj.Base.Shape.Wires[0]
                    if FreeCAD.GuiUp:
                        obj.Base.ViewObject.hide()
        if baseWire:
            if baseWire.isClosed():
                if FreeCAD.GuiUp:
                    obj.Base.ViewObject.hide()
                edges = Part.__sortEdges__(baseWire.Edges)
                ln = len(edges)
                obj.Angles    = ArchRoof.adjust_list_len(angles, ln, angles[0])
                obj.Runs      = ArchRoof.adjust_list_len(run, ln, run[0])
                obj.IdRel     = ArchRoof.adjust_list_len(idrel, ln, idrel[0])
                obj.Thickness = ArchRoof.adjust_list_len(thickness, ln, thickness[0])
                obj.Overhang  = ArchRoof.adjust_list_len(overhang, ln, overhang[0])
    obj.Face = facenr
    return obj


def makeSectionPlane(objectslist=None,name=None):

    """makeSectionPlane([objectslist],[name]) : Creates a Section plane objects including the
    given objects. If no object is given, the whole document will be considered."""

    import ArchSectionPlane
    import Draft
    import WorkingPlane
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("App::FeaturePython","Section")
    obj.Label = name if name else translate("Arch","Section")
    ArchSectionPlane._SectionPlane(obj)
    if FreeCAD.GuiUp:
        ArchSectionPlane._ViewProviderSectionPlane(obj.ViewObject)
    if objectslist:
        obj.Objects = objectslist
        bb = FreeCAD.BoundBox()
        for o in Draft.get_group_contents(objectslist):
            if hasattr(o,"Shape") and hasattr(o.Shape,"BoundBox"):
                bb.add(o.Shape.BoundBox)
        obj.Placement = WorkingPlane.get_working_plane().get_placement()
        obj.Placement.Base = bb.Center
        if FreeCAD.GuiUp:
            margin = bb.XLength*0.1
            obj.ViewObject.DisplayLength = bb.XLength+margin
            obj.ViewObject.DisplayHeight = bb.YLength+margin
    return obj


def makeSite(objectslist=None,baseobj=None,name=None):

    '''makeBuilding([objectslist],[baseobj],[name]): creates a site including the
    objects from the given list.'''

    import ArchSite
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    import Part
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Site")
    obj.Label = name if name else translate("Arch","Site")
    ArchSite._Site(obj)
    if FreeCAD.GuiUp:
        ArchSite._ViewProviderSite(obj.ViewObject)
    if objectslist:
        obj.Group = objectslist
    if baseobj:
        import Part
        if isinstance(baseobj,Part.Shape):
            obj.Shape = baseobj
        else:
            obj.Terrain = baseobj
    return obj


def makeSpace(objects=None,baseobj=None,name=None):

    """makeSpace([objects],[baseobj],[name]): Creates a space object from the given objects.
    Objects can be one document object, in which case it becomes the base shape of the space
    object, or a list of selection objects as got from getSelectionEx(), or a list of tuples
    (object, subobjectname)"""

    import ArchSpace
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Space")
    obj.Label = name if name else translate("Arch","Space")
    ArchSpace._Space(obj)
    if FreeCAD.GuiUp:
        ArchSpace._ViewProviderSpace(obj.ViewObject)
    if baseobj:
        objects = baseobj
    if objects:
        if not isinstance(objects,list):
            objects = [objects]
        if len(objects) == 1:
            obj.Base = objects[0]
            if FreeCAD.GuiUp:
                objects[0].ViewObject.hide()
        else:
            obj.Proxy.addSubobjects(obj,objects)
    return obj


def makeStairs(baseobj=None,length=None,width=None,height=None,steps=None,name=None):

    """makeStairs([baseobj],[length],[width],[height],[steps],[name]): creates a Stairs
    objects with given attributes."""

    import ArchStairs
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return

    stairs = []
    additions = []
    label = name if name else translate("Arch","Stairs")

    def setProperty(obj,length,width,height,steps):
        if length:
            obj.Length = length
        else:
            obj.Length = params.get_param_arch("StairsLength")
        if width:
            obj.Width = width
        else:
            obj.Width = params.get_param_arch("StairsWidth")
        if height:
            obj.Height = height
        else:
            obj.Height = params.get_param_arch("StairsHeight")
        if steps:
            obj.NumberOfSteps = steps
        obj.Structure = "Massive"
        obj.StructureThickness = 150
        obj.DownSlabThickness = 150
        obj.UpSlabThickness = 150

        obj.RailingOffsetLeft = 60
        obj.RailingOffsetRight = 60
        obj.RailingHeightLeft = 900
        obj.RailingHeightRight = 900

    if baseobj:
        if not isinstance(baseobj,list):
            baseobj = [baseobj]
        lenSelection = len(baseobj)
        if lenSelection > 1:
            stair = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Stairs")
            stair.Label = label
            ArchStairs._Stairs(stair)
            stairs.append(stair)
            stairs[0].Label = label
            i = 1
        else:
            i = 0
        for baseobjI in baseobj:
            stair = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Stairs")
            stair.Label = label
            ArchStairs._Stairs(stair)
            stairs.append(stair)
            stairs[i].Label = label
            stairs[i].Base = baseobjI

            if len(baseobjI.Shape.Edges) > 1:
                stepsI = 1                              #'landing' if 'multi-edges' currently
            elif steps:
                stepsI = steps
            else:
                stepsI = 20
            setProperty(stairs[i],None,width,height,stepsI)

            if i > 1:
                additions.append(stairs[i])
                stairs[i].LastSegment = stairs[i-1]
            else:
                if len(stairs) > 1:                     # i.e. length >1, have a 'master' staircase created
                    stairs[0].Base = stairs[1]
            i += 1
        if lenSelection > 1:
            stairs[0].Additions = additions
    else:
        obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Stairs")
        obj.Label = label
        ArchStairs._Stairs(obj)
        setProperty(obj,length,width,height,steps)
        stairs.append(obj)
    if FreeCAD.GuiUp:
        if baseobj:
            for stair in stairs:
                ArchStairs._ViewProviderStairs(stair.ViewObject)
        else:
            ArchStairs._ViewProviderStairs(obj.ViewObject)
    if stairs:
        for stair in stairs:
            stair.recompute()
        makeRailing(stairs)
        # return stairs - all other functions expect one object as return value
        return stairs[0]
    else:
        obj.recompute()
        return obj


def makeRailing(stairs):

    "simple make Railing function"

    import ArchPipe

    def makeRailingLorR(stairs,side="L"):
        for stair in reversed(stairs):
            if side == "L":
                outlineLR = stair.OutlineLeft
                outlineLRAll = stair.OutlineLeftAll
                stairRailingLR = "RailingLeft"
            elif side == "R":
                outlineLR = stair.OutlineRight
                outlineLRAll = stair.OutlineRightAll
                stairRailingLR = "RailingRight"
            if outlineLR or outlineLRAll:
                lrRail = makePipe(baseobj=None,diameter=0,length=0,placement=None,name=translate("Arch","Railing"))
                if outlineLRAll:
                    setattr(stair, stairRailingLR, lrRail)
                    break
                elif outlineLR:
                    setattr(stair, stairRailingLR, lrRail)

    if stairs is None:
        sel = FreeCADGui.Selection.getSelection()
        sel0 = sel[0]
        stairs = []
        # TODO currently consider 1st selected object, then would tackle multiple objects?
        if Draft.getType(sel[0]) == "Stairs":
            stairs.append(sel0)
            if Draft.getType(sel0.Base) == "Stairs":
                stairs.append(sel0.Base)
                additions = sel0.Additions
                for additionsI in additions:
                    if Draft.getType(additionsI) == "Stairs":
                        stairs.append(additionsI)
            else:
                stairs.append(sel[0])
        else:
            print("No Stairs object selected")
            return

    makeRailingLorR(stairs,"L")
    makeRailingLorR(stairs,"R")


def makeTruss(baseobj=None,name=None):

    """
    makeTruss([baseobj],[name]): Creates a space object from the given object (a line)
    """

    import ArchTruss
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Truss")
    obj.Label = name if name else translate("Arch","Truss")
    ArchTruss.Truss(obj)
    if FreeCAD.GuiUp:
        ArchTruss.ViewProviderTruss(obj.ViewObject)
    if baseobj:
        obj.Base = baseobj
        if FreeCAD.GuiUp:
            baseobj.ViewObject.hide()
    return obj


def makeWall(baseobj=None,height=None,length=None,width=None,align=None,face=None,name=None):
    """Create a wall based on a given object, and returns the generated wall.

    TODO: It is unclear what defines which units this function uses.

    Parameters
    ----------
    baseobj: <Part::PartFeature>, optional
        The base object with which to build the wall. This can be a sketch, a
        draft object, a face, or a solid. It can also be left as None.
    height: float, optional
        The height of the wall.
    length: float, optional
        The length of the wall. Not used if the wall is based off an object.
        Will use Arch default if left empty.
    width: float, optional
        The width of the wall. Not used if the base object is a face.  Will use
        Arch default if left empty.
    align: str, optional
        Either "Center", "Left", or "Right". Effects the alignment of the wall
        on its baseline.
    face: int, optional
        The index number of a face on the given baseobj, to base the wall on.
    name: str, optional
        The name to give to the created wall.

    Returns
    -------
    <Part::FeaturePython>
        Returns the generated wall.

    Notes
    -----
    Creates a new <Part::FeaturePython> object, and turns it into a parametric wall
    object. This <Part::FeaturePython> object does not yet have any shape.

    The wall then uses the baseobj.Shape as the basis to extrude out a wall shape,
    giving the new <Part::FeaturePython> object a shape.

    It then hides the original baseobj.
    """

    import ArchWall
    import Draft
    from draftutils import params
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Wall")
    if name:
        obj.Label = name
    else:
        obj.Label = translate("Arch","Wall")
    ArchWall._Wall(obj)
    if FreeCAD.GuiUp:
        ArchWall._ViewProviderWall(obj.ViewObject)
    if baseobj:
        if hasattr(baseobj,'Shape') or baseobj.isDerivedFrom("Mesh::Feature"):
            obj.Base = baseobj
        else:
            FreeCAD.Console.PrintWarning(str(translate("Arch","Walls can only be based on Part or Mesh objects")))
    if face:
        obj.Face = face
    if length:
        obj.Length = length
    if width:
        obj.Width = width
    else:
        obj.Width = params.get_param_arch("WallWidth")
    if height:
        obj.Height = height
    else:
        obj.Height = params.get_param_arch("WallHeight")
    if align:
        obj.Align = align
    else:
        obj.Align = ["Center","Left","Right"][params.get_param_arch("WallAlignment")]
    if obj.Base and FreeCAD.GuiUp:
        if Draft.getType(obj.Base) != "Space":
            obj.Base.ViewObject.hide()
    return obj


def joinWalls(walls,delete=False):
    """Join the given list of walls into one sketch-based wall.

    Take the first wall in the list, and adds on the other walls in the list.
    Return the modified first wall.

    Setting delete to True, will delete the other walls. Only join walls
    if the walls have the same width, height and alignment.

    Parameters
    ----------
    walls: list of <Part::FeaturePython>
        List containing the walls to add to the first wall in the list. Walls must
        be based off a base object.
    delete: bool, optional
        If True, deletes the other walls in the list.

    Returns
    -------
    <Part::FeaturePython>
    """

    import Part
    import Draft
    import ArchWall
    if not walls:
        return None
    if not isinstance(walls,list):
        walls = [walls]
    if not ArchWall.areSameWallTypes(walls):
        return None
    deleteList = []
    base = walls.pop()
    if base.Base:
        if base.Base.Shape.Faces:
            return None
        # Use ArchSketch if SketchArch add-on is present
        if Draft.getType(base.Base) == "ArchSketch":
            sk = base.Base
        else:
            try:
                import ArchSketchObject
                newSk=ArchSketchObject.makeArchSketch()
            except:
                if Draft.getType(base.Base) != "Sketcher::SketchObject":
                    newSk=FreeCAD.ActiveDocument.addObject("Sketcher::SketchObject","WallTrace")
                else:
                    newSk=None
            if newSk:
                sk = Draft.makeSketch(base.Base,autoconstraints=True, addTo=newSk)
                base.Base = sk
            else:
                sk = base.Base
    for w in walls:
        if w.Base:
            if not w.Base.Shape.Faces:
                for e in w.Base.Shape.Edges:
                    l = e.Curve
                    if isinstance(l,Part.Line):
                        l = Part.LineSegment(e.Vertexes[0].Point,e.Vertexes[-1].Point)
                    sk.addGeometry(l)
                    deleteList.append(w.Name)
    if delete:
        for n in deleteList:
            FreeCAD.ActiveDocument.removeObject(n)
    FreeCAD.ActiveDocument.recompute()
    base.ViewObject.show()
    return base


def makeWindow(baseobj=None,width=None,height=None,parts=None,name=None):

    '''makeWindow(baseobj,[width,height,parts,name]): creates a window based on the
    given base 2D object (sketch or draft).'''

    import ArchWindow
    import Draft
    from DraftGui import todo
    if not FreeCAD.ActiveDocument:
        FreeCAD.Console.PrintError("No active document. Aborting\n")
        return
    if baseobj:
        if Draft.getType(baseobj) == "Window":
            obj = Draft.clone(baseobj)
            return obj
    obj = FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Window")
    ArchWindow._Window(obj)
    if name:
        obj.Label = name
    else:
        obj.Label = translate("Arch","Window")
    if FreeCAD.GuiUp:
        ArchWindow._ViewProviderWindow(obj.ViewObject)
    if width:
        obj.Width = width
    if height:
        obj.Height = height
    if baseobj:
        obj.Normal = baseobj.Placement.Rotation.multVec(FreeCAD.Vector(0,0,-1))
        obj.Base = baseobj
    if parts is not None:
        obj.WindowParts = parts
    else:
        if baseobj:
            if baseobj.getLinkedObject().isDerivedFrom("Part::Part2DObject"):
                # create default component
                if baseobj.Shape.Wires:
                    tp = "Frame"
                    if len(baseobj.Shape.Wires) == 1:
                        tp = "Solid panel"
                    i = 0
                    ws = ''
                    for w in baseobj.Shape.Wires:
                        if w.isClosed():
                            if ws: ws += ","
                            ws += "Wire" + str(i)
                            i += 1
                    obj.WindowParts = ["Default",tp,ws,"1","0"]
            else:
                # bind properties from base obj if existing
                for prop in ["Height","Width","Subvolume","Tag","Description","Material"]:
                    for p in baseobj.PropertiesList:
                        if (p == prop) or p.endswith("_"+prop):
                            obj.setExpression(prop, baseobj.Name+"."+p)

    if obj.Base and FreeCAD.GuiUp:
        obj.Base.ViewObject.DisplayMode = "Wireframe"
        obj.Base.ViewObject.hide()
        todo.delay(ArchWindow.recolorize,[obj.Document.Name,obj.Name])
    return obj



# SPDX-License-Identifier: LGPL-2.1-or-later

# /***************************************************************************
# *   Copyright (c) 2024 Mario Passaglia <mpassaglia[at]cbc.uba.ar>         *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# **************************************************************************

__title__ = "FreeCAD FEM calculix constraint body heat source"
__author__ = "Mario Passaglia"
__url__ = "https://www.freecad.org"

import FreeCAD
import itertools


def get_analysis_types():
    return ["thermomech"]


def get_sets_name():
    return "constraints_bodyheatsource_element_sets"


def get_constraint_title():
    return "Body Heat Source Constraints"


def get_before_write_meshdata_constraint():
    return ""


def get_after_write_meshdata_constraint():
    return ""


def get_before_write_constraint():
    return ""


def get_after_write_constraint():
    return ""


def write_meshdata_constraint(f, femobj, bodyheatsource_obj, ccxwriter):
    f.write(f"*ELSET,ELSET={bodyheatsource_obj.Name}\n")
    if isinstance(femobj["FEMElements"], str):
        f.write("{}\n".format(femobj["FEMElements"]))
    else:
        for e in femobj["FEMElements"]:
            f.write(f"{e},\n")


def write_constraint(f, femobj, bodyheatsource_obj, ccxwriter):

    # floats read from ccx should use {:.13G}, see comment in writer module
    # search referenced material
    ref = bodyheatsource_obj.References[0]
    ref_feat = ref[0]
    ref_sub_obj = ref[1][0]
    density = None
    for mat in ccxwriter.member.mats_linear:
        mat_ref = [
            *itertools.chain(*[itertools.product([i[0]], i[1]) for i in mat["Object"].References])
        ]
        if (ref_feat, ref_sub_obj) in mat_ref:
            density = FreeCAD.Units.Quantity(mat["Object"].Material["Density"])
            break

    if not density:
        # search material without references
        for mat in ccxwriter.member.mats_linear:
            if not mat["Object"].References:
                density = FreeCAD.Units.Quantity(mat["Object"].Material["Density"])
                break

    # get data from the bodyheatsource_obj (DissipationRate is in power per unit mass)
    if bodyheatsource_obj.Mode == "Dissipation Rate":
        heat = bodyheatsource_obj.DissipationRate * density
    elif bodyheatsource_obj.Mode == "Total Power":
        volume = ref_feat.getSubObject(ref_sub_obj).Volume
        heat = bodyheatsource_obj.TotalPower / FreeCAD.Units.Quantity(volume, "mm^3")
    # write to file
    f.write("*DFLUX\n")
    f.write("{},BF,{:.13G}\n".format(bodyheatsource_obj.Name, heat.getValueAs("t/(mm*s^3)").Value))
    f.write("\n")



# SPDX-License-Identifier: LGPL-2.1-or-later
# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2024 FreeCAD Project Association                        *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

""" Defines a QWidget-derived class for displaying the single-addon buttons. """

from enum import Enum, auto

try:
    import FreeCAD

    translate = FreeCAD.Qt.translate
except ImportError:
    FreeCAD = None

    def translate(_: str, text: str):
        return text


# Get whatever version of PySide we can
try:
    import PySide  # Use the FreeCAD wrapper
except ImportError:
    try:
        import PySide6  # Outside FreeCAD, try Qt6 first

        PySide = PySide6
    except ImportError:
        import PySide2  # Fall back to Qt5 (if this fails, Python will kill this module's import)

        PySide = PySide2

from PySide import QtCore, QtGui, QtWidgets


class ButtonBarDisplayMode(Enum):
    TextOnly = auto()
    IconsOnly = auto()
    TextAndIcons = auto()


class WidgetAddonButtons(QtWidgets.QWidget):
    def __init__(self, parent: QtWidgets.QWidget = None):
        super().__init__(parent)
        self.display_mode = ButtonBarDisplayMode.TextAndIcons
        self._setup_ui()
        self._set_icons()
        self.retranslateUi(None)

    def set_display_mode(self, mode: ButtonBarDisplayMode):
        """NOTE: Not really implemented yet -- TODO: Implement this functionality"""
        if mode == self.display_mode:
            return
        self._setup_ui()
        self._set_icons()
        self.retranslateUi(None)

    def _setup_ui(self):
        if self.layout():
            self.setLayout(None)  # TODO: Check this
        self.horizontal_layout = QtWidgets.QHBoxLayout()
        self.horizontal_layout.setContentsMargins(0, 0, 0, 0)
        self.back = QtWidgets.QToolButton(self)
        self.install = QtWidgets.QPushButton(self)
        self.uninstall = QtWidgets.QPushButton(self)
        self.enable = QtWidgets.QPushButton(self)
        self.disable = QtWidgets.QPushButton(self)
        self.update = QtWidgets.QPushButton(self)
        self.run_macro = QtWidgets.QPushButton(self)
        self.change_branch = QtWidgets.QPushButton(self)
        self.check_for_update = QtWidgets.QPushButton(self)
        self.horizontal_layout.addWidget(self.back)
        self.horizontal_layout.addStretch()
        self.horizontal_layout.addWidget(self.check_for_update)
        self.horizontal_layout.addWidget(self.install)
        self.horizontal_layout.addWidget(self.uninstall)
        self.horizontal_layout.addWidget(self.enable)
        self.horizontal_layout.addWidget(self.disable)
        self.horizontal_layout.addWidget(self.update)
        self.horizontal_layout.addWidget(self.run_macro)
        self.horizontal_layout.addWidget(self.change_branch)
        self.setLayout(self.horizontal_layout)

    def set_show_back_button(self, show: bool) -> None:
        self.back.setVisible(show)

    def _set_icons(self):
        self.back.setIcon(QtGui.QIcon.fromTheme("back", QtGui.QIcon(":/icons/button_left.svg")))

    def retranslateUi(self, _):
        self.check_for_update.setText(translate("AddonsInstaller", "Check for update"))
        self.install.setText(translate("AddonsInstaller", "Install"))
        self.uninstall.setText(translate("AddonsInstaller", "Uninstall"))
        self.disable.setText(translate("AddonsInstaller", "Disable"))
        self.enable.setText(translate("AddonsInstaller", "Enable"))
        self.update.setText(translate("AddonsInstaller", "Update"))
        self.run_macro.setText(translate("AddonsInstaller", "Run"))
        self.change_branch.setText(translate("AddonsInstaller", "Change branch..."))
        self.back.setToolTip(translate("AddonsInstaller", "Return to package list"))



# -*- coding: utf8 -*-

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2017 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""The BIM TD View command"""


import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate


class BIM_TDView:
    def GetResources(self):
        return {
            "Pixmap": "techdraw-ArchView",
            "MenuText": QT_TRANSLATE_NOOP("BIM_TDView", "View"),
            "ToolTip": QT_TRANSLATE_NOOP(
                "BIM_TDView",
                "Creates a TechDraw view from a section plane or 2D objects",
            ),
        }

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        import Draft
        sections = []
        page = None
        drafts = []
        for obj in FreeCADGui.Selection.getSelection():
            t = Draft.getType(obj)
            if t == "SectionPlane":
                sections.append(obj)
            elif t == "TechDraw::DrawPage":
                page = obj
            else:
                drafts.append(obj)
        if not page:
            pages = FreeCAD.ActiveDocument.findObjects(Type="TechDraw::DrawPage")
            if pages:
                page = pages[0]
        if (not page) or ((not sections) and (not drafts)):
            FreeCAD.Console.PrintError(
                translate(
                    "BIM",
                    "No section view or Draft objects selected, or no page selected, or no page found in document",
                )
                + "\n"
            )
            return
        FreeCAD.ActiveDocument.openTransaction("Create view")
        for section in sections:
            view = FreeCAD.ActiveDocument.addObject(
                "TechDraw::DrawViewArch", "ArchView"
            )
            view.Label = section.Label
            view.Source = section
            page.addView(view)
            if page.Scale:
                view.Scale = page.Scale
        for draft in drafts:
            view = FreeCAD.ActiveDocument.addObject(
                "TechDraw::DrawViewDraft", "DraftView"
            )
            view.Label = draft.Label
            view.Source = draft
            page.addView(view)
            if page.Scale:
                view.Scale = page.Scale
        FreeCAD.ActiveDocument.commitTransaction()
        FreeCAD.ActiveDocument.recompute()


FreeCADGui.addCommand("BIM_TDView", BIM_TDView())



# SPDX-License-Identifier: LGPL-2.1-or-later
# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2022-2024 The FreeCAD Project Association AISBL         *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

from dataclasses import dataclass
from enum import Enum, auto
import os
from typing import Optional

try:
    import FreeCAD

    translate = FreeCAD.Qt.translate
except ImportError:
    FreeCAD = None

    def translate(_: str, text: str):
        return text


# Get whatever version of PySide we can
try:
    import PySide  # Use the FreeCAD wrapper
except ImportError:
    try:
        import PySide6  # Outside FreeCAD, try Qt6 first

        PySide = PySide6
    except ImportError:
        import PySide2  # Fall back to Qt5 (if this fails, Python will kill this module's import)

        PySide = PySide2

from PySide import QtCore, QtWidgets

from .addonmanager_widget_addon_buttons import WidgetAddonButtons
from .addonmanager_widget_readme_browser import WidgetReadmeBrowser
from .addonmanager_colors import warning_color_string, attention_color_string, bright_color_string


class MessageType(Enum):
    Message = auto()
    Warning = auto()
    Error = auto()


@dataclass
class UpdateInformation:
    unchecked: bool = True
    check_in_progress: bool = False
    update_available: bool = False
    detached_head: bool = False
    version: str = ""
    tag: str = ""
    branch: Optional[str] = None


@dataclass
class WarningFlags:
    obsolete: bool = False
    python2: bool = False
    required_freecad_version: Optional[str] = None
    non_osi_approved = False
    non_fsf_libre = False


class PackageDetailsView(QtWidgets.QWidget):
    """The view class for the package details"""

    def __init__(self, parent: QtWidgets.QWidget = None):
        super().__init__(parent)
        self.button_bar = None
        self.readme_browser = None
        self.message_label = None
        self.location_label = None
        self.installed = False
        self.disabled = False
        self.update_info = UpdateInformation()
        self.warning_flags = WarningFlags()
        self.installed_version = None
        self.installed_branch = None
        self.installed_timestamp = None
        self.can_disable = True
        self._setup_ui()

    def _setup_ui(self):
        self.vertical_layout = QtWidgets.QVBoxLayout(self)
        self.button_bar = WidgetAddonButtons(self)
        self.readme_browser = WidgetReadmeBrowser(self)
        self.message_label = QtWidgets.QLabel(self)
        self.location_label = QtWidgets.QLabel(self)
        self.location_label.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
        self.vertical_layout.addWidget(self.button_bar)
        self.vertical_layout.addWidget(self.message_label)
        self.vertical_layout.addWidget(self.location_label)
        self.vertical_layout.addWidget(self.readme_browser)
        self.button_bar.hide()  # Start with no bar

    def set_location(self, location: Optional[str]):
        if location is not None:
            text = (
                translate("AddonsInstaller", "Installation location")
                + ": "
                + os.path.normpath(location)
            )
            self.location_label.setText(text)
            self.location_label.show()
        else:
            self.location_label.hide()

    def set_installed(
        self,
        installed: bool,
        on_date: Optional[str] = None,
        version: Optional[str] = None,
        branch: Optional[str] = None,
    ):
        self.installed = installed
        self.installed_timestamp = on_date
        self.installed_version = version
        self.installed_branch = branch
        if not self.installed:
            self.set_location(None)
        self._sync_ui_state()

    def set_update_available(self, info: UpdateInformation):
        self.update_info = info
        self._sync_ui_state()

    def set_disabled(self, disabled: bool):
        self.disabled = disabled
        self._sync_ui_state()

    def allow_disabling(self, allow: bool):
        self.can_disable = allow
        self._sync_ui_state()

    def allow_running(self, show: bool):
        self.button_bar.run_macro.setVisible(show)

    def set_warning_flags(self, flags: WarningFlags):
        self.warning_flags = flags
        self._sync_ui_state()

    def set_new_disabled_status(self, disabled: bool):
        """If the user just changed the enabled/disabled state of the addon, display a message
        indicating that will not take place until restart. Do not call except in a case of a
        state change during this run."""

        if disabled:
            message = translate(
                "AddonsInstaller", "This Addon will be disabled next time you restart FreeCAD."
            )
        else:
            message = translate(
                "AddonsInstaller", "This Addon will be enabled next time you restart FreeCAD."
            )
        self.message_label.setText(f"<h3>{message}</h3>")
        self.message_label.setStyleSheet("color:" + attention_color_string())

    def set_new_branch(self, branch: str):
        """If the user just changed branches, update the message to show that a restart is
        needed."""
        message_string = "<h3>"
        message_string += translate(
            "AddonsInstaller", "Changed to branch '{}' -- please restart to use Addon."
        ).format(branch)
        message_string += "</h3>"
        self.message_label.setText(message_string)
        self.message_label.setStyleSheet("color:" + attention_color_string())

    def set_updated(self):
        """If the user has just updated the addon but not yet restarted, show an indication that
        we are awaiting a restart."""
        message = translate(
            "AddonsInstaller", "This Addon has been updated. Restart FreeCAD to see changes."
        )
        self.message_label.setText(f"<h3>{message}</h3>")
        self.message_label.setStyleSheet("color:" + attention_color_string())

    def _sync_ui_state(self):
        self._sync_button_state()
        self._create_status_label_text()

    def _sync_button_state(self):
        self.button_bar.install.setVisible(not self.installed)
        self.button_bar.uninstall.setVisible(self.installed)
        if not self.installed:
            self.button_bar.disable.hide()
            self.button_bar.enable.hide()
            self.button_bar.update.hide()
            self.button_bar.check_for_update.hide()
        else:
            self.button_bar.update.setVisible(self.update_info.update_available)
            if self.update_info.detached_head:
                self.button_bar.check_for_update.hide()
            else:
                self.button_bar.check_for_update.setVisible(not self.update_info.update_available)
            if self.can_disable:
                self.button_bar.enable.setVisible(self.disabled)
                self.button_bar.disable.setVisible(not self.disabled)
            else:
                self.button_bar.enable.hide()
                self.button_bar.disable.hide()

    def _create_status_label_text(self):
        if self.installed:
            installation_details = self._get_installation_details_string()
            update_details = self._get_update_status_string()
            message_text = f"{installation_details} {update_details}"
            if self.disabled:
                message_text += " [" + translate("AddonsInstaller", "Disabled") + "]"
            self.message_label.setText(f"<h3>{message_text}</h3>")
            if self.disabled:
                self.message_label.setStyleSheet("color:" + warning_color_string())
            elif self.update_info.update_available:
                self.message_label.setStyleSheet("color:" + attention_color_string())
            else:
                self.message_label.setStyleSheet("color:" + bright_color_string())
            self.message_label.show()
        elif self._there_are_warnings_to_show():
            warnings = self._get_warning_string()
            self.message_label.setText(f"<h3>{warnings}</h3>")
            self.message_label.setStyleSheet("color:" + warning_color_string())
            self.message_label.show()
        else:
            self.message_label.hide()

    def _get_installation_details_string(self) -> str:
        version = self.installed_version
        date = ""
        installed_version_string = ""
        if self.installed_timestamp:
            date = QtCore.QLocale().toString(
                QtCore.QDateTime.fromSecsSinceEpoch(int(round(self.installed_timestamp, 0))),
                QtCore.QLocale.ShortFormat,
            )
        if version and date:
            installed_version_string += (
                translate("AddonsInstaller", "Version {version} installed on {date}").format(
                    version=version, date=date
                )
                + ". "
            )
        elif version:
            installed_version_string += (
                translate("AddonsInstaller", "Version {version} installed") + "."
            ).format(version=version)
        elif date:
            installed_version_string += (
                translate("AddonsInstaller", "Installed on {date}") + "."
            ).format(date=date)
        else:
            installed_version_string += translate("AddonsInstaller", "Installed") + "."
        return installed_version_string

    def _get_update_status_string(self) -> str:
        if self.update_info.check_in_progress:
            return translate("AddonsInstaller", "Update check in progress") + "."
        elif self.update_info.unchecked:
            return ""
        if self.update_info.detached_head:
            return (
                translate(
                    "AddonsInstaller", "Git tag '{}' checked out, no updates possible"
                ).format(self.update_info.tag)
                + "."
            )
        if self.update_info.update_available:
            if self.installed_branch and self.update_info.branch:
                if self.installed_branch != self.update_info.branch:
                    return (
                        translate(
                            "AddonsInstaller", "Currently on branch {}, name changed to {}"
                        ).format(self.installed_branch, self.update_info.branch)
                        + "."
                    )
                if self.update_info.version:
                    return (
                        translate(
                            "AddonsInstaller",
                            "Currently on branch {}, update available to version {}",
                        ).format(self.installed_branch, str(self.update_info.version).strip())
                        + "."
                    )
                return translate("AddonsInstaller", "Update available") + "."
            if self.update_info.version:
                return (
                    translate("AddonsInstaller", "Update available to version {}").format(
                        str(self.update_info.version).strip()
                    )
                    + "."
                )
            return translate("AddonsInstaller", "Update available") + "."
        return translate("AddonsInstaller", "This is the latest version available") + "."

    def _there_are_warnings_to_show(self) -> bool:
        if self.disabled:
            return True
        if (
            self.warning_flags.obsolete
            or self.warning_flags.python2
            or self.warning_flags.required_freecad_version
        ):
            return True
        return False  # TODO: Someday support optional warnings on license types

    def _get_warning_string(self) -> str:
        if self.installed and self.disabled:
            return translate(
                "AddonsInstaller",
                "WARNING: This addon is currently installed, but disabled. Use the 'enable' "
                "button to re-enable.",
            )
        if self.warning_flags.obsolete:
            return translate("AddonsInstaller", "WARNING: This addon is obsolete")
        if self.warning_flags.python2:
            return translate("AddonsInstaller", "WARNING: This addon is Python 2 only")
        if self.warning_flags.required_freecad_version:
            return translate("AddonsInstaller", "WARNING: This addon requires FreeCAD {}").format(
                self.warning_flags.required_freecad_version
            )
        return ""



# SPDX-License-Identifier: LGPL-2.1-or-later
# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2022-2023 FreeCAD Project Association                   *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************
import datetime
import sys
import unittest
from datetime import date
from unittest import TestCase
from unittest.mock import MagicMock, patch

sys.path.append("../..")

import addonmanager_cache as cache
from AddonManagerTest.app.mocks import MockPref, MockExists


class TestCache(TestCase):
    @patch("addonmanager_freecad_interface.getUserCachePath")
    @patch("addonmanager_freecad_interface.ParamGet")
    @patch("os.remove", MagicMock())
    @patch("os.makedirs", MagicMock())
    def test_local_cache_needs_update(self, param_mock: MagicMock, cache_mock: MagicMock):
        cache_mock.return_value = ""
        param_mock.return_value = MockPref()
        default_prefs = {
            "UpdateFrequencyComboEntry": 0,
            "LastCacheUpdate": "2000-01-01",
            "CustomRepoHash": "",
            "CustomRepositories": "",
        }
        today = date.today().isoformat()
        yesterday = (date.today() - datetime.timedelta(1)).isoformat()

        # Organize these as subtests because of all the patching that has to be done: once we are in this function,
        # the patch is complete, and we can just modify the return values of the fakes one by one
        tests = (
            {
                "case": "No existing cache",
                "files_that_exist": [],
                "prefs_to_set": {},
                "expect": True,
            },
            {
                "case": "Last cache update was interrupted",
                "files_that_exist": ["CACHE_UPDATE_INTERRUPTED"],
                "prefs_to_set": {},
                "expect": True,
            },
            {
                "case": "Cache exists and updating is blocked",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {},
                "expect": False,
            },
            {
                "case": "Daily updates set and last update was long ago",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"UpdateFrequencyComboEntry": 1},
                "expect": True,
            },
            {
                "case": "Daily updates set and last update was today",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"UpdateFrequencyComboEntry": 1, "LastCacheUpdate": today},
                "expect": False,
            },
            {
                "case": "Daily updates set and last update was yesterday",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"UpdateFrequencyComboEntry": 1, "LastCacheUpdate": yesterday},
                "expect": True,
            },
            {
                "case": "Weekly updates set and last update was long ago",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"UpdateFrequencyComboEntry": 1},
                "expect": True,
            },
            {
                "case": "Weekly updates set and last update was yesterday",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"UpdateFrequencyComboEntry": 2, "LastCacheUpdate": yesterday},
                "expect": False,
            },
            {
                "case": "Custom repo list changed",
                "files_that_exist": ["AddonManager"],
                "prefs_to_set": {"CustomRepositories": "NewRepo"},
                "expect": True,
            },
        )
        for test_case in tests:
            with self.subTest(test_case["case"]):
                case_prefs = default_prefs
                for pref, setting in test_case["prefs_to_set"].items():
                    case_prefs[pref] = setting
                param_mock.return_value.set_prefs(case_prefs)
                exists_mock = MockExists(test_case["files_that_exist"])
                with patch("os.path.exists", exists_mock.exists):
                    if test_case["expect"]:
                        self.assertTrue(cache.local_cache_needs_update())
                    else:
                        self.assertFalse(cache.local_cache_needs_update())


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf8 -*-

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2017 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""The BIM MoveView command"""

import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP


class BIM_MoveView:

    def GetResources(self):
        return {
            "Pixmap": "BIM_MoveView",
            "MenuText": QT_TRANSLATE_NOOP("BIM_MoveView", "Move view..."),
            "ToolTip": QT_TRANSLATE_NOOP(
                "BIM_MoveView", "Moves this view to an existing page"
            ),
        }

    def Activated(self):
        self.selection = FreeCADGui.Selection.getSelection()
        self.pages = FreeCAD.ActiveDocument.findObjects(Type="TechDraw::DrawPage")
        self.labels = [obj.Label for obj in self.pages]
        FreeCADGui.draftToolBar.sourceCmd = self
        FreeCADGui.draftToolBar.popupMenu(self.labels)

    def proceed(self, labelname):
        FreeCADGui.draftToolBar.sourceCmd = None
        if labelname in self.labels:
            page = self.pages[self.labels.index(labelname)]
            for view in self.selection:
                for p in view.InList:
                    if hasattr(p, "Views") and view in p.Views:
                        p.removeView(view)
                x = view.X
                y = view.Y
                page.addView(view)
                FreeCAD.ActiveDocument.recompute()
                view.X = x
                view.Y = y


FreeCADGui.addCommand("BIM_MoveView", BIM_MoveView())



# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2024 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""Misc Arch util commands"""


import os
import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate
PARAMS = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/BIM")




class Arch_Add:
    "the Arch Add command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_Add',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Add","Add component"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Add","Adds the selected components to the active object")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and len(FreeCADGui.Selection.getSelection()) > 1

    def Activated(self):
        import Draft
        import Arch
        sel = FreeCADGui.Selection.getSelection()
        if Draft.getType(sel[-1]) == "Space":
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Add space boundary"))
            FreeCADGui.addModule("Arch")
            FreeCADGui.doCommand("Arch.addSpaceBoundaries( FreeCAD.ActiveDocument."+sel[-1].Name+", FreeCADGui.Selection.getSelectionEx() )")
        else:
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Grouping"))
            if not Arch.mergeCells(sel):
                host = sel.pop()
                ss = "["
                for o in sel:
                    if len(ss) > 1:
                        ss += ","
                    ss += "FreeCAD.ActiveDocument."+o.Name
                ss += "]"
                FreeCADGui.addModule("Arch")
                FreeCADGui.doCommand("Arch.addComponents("+ss+",FreeCAD.ActiveDocument."+host.Name+")")
        FreeCAD.ActiveDocument.commitTransaction()
        FreeCAD.ActiveDocument.recompute()


class Arch_Remove:
    "the Arch Add command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_Remove',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Remove","Remove component"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Remove","Remove the selected components from their parents, or create a hole in a component")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Draft
        sel = FreeCADGui.Selection.getSelection()
        if Draft.getType(sel[-1]) == "Space":
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Remove space boundary"))
            FreeCADGui.addModule("Arch")
            FreeCADGui.doCommand("Arch.removeSpaceBoundaries( FreeCAD.ActiveDocument."+sel[-1].Name+", FreeCADGui.Selection.getSelection() )")
        else:
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Ungrouping"))
            if len(sel) > 1:
                host = sel.pop()
                ss = "["
                for o in sel:
                    if len(ss) > 1:
                        ss += ","
                    ss += "FreeCAD.ActiveDocument."+o.Name
                ss += "]"
                FreeCADGui.addModule("Arch")
                FreeCADGui.doCommand("Arch.removeComponents("+ss+",FreeCAD.ActiveDocument."+host.Name+")")
            else:
                FreeCADGui.addModule("Arch")
                FreeCADGui.doCommand("Arch.removeComponents(FreeCAD.ActiveDocument."+sel[0].Name+")")
        FreeCAD.ActiveDocument.commitTransaction()
        FreeCAD.ActiveDocument.recompute()


class Arch_SplitMesh:
    "the Arch SplitMesh command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_SplitMesh',
                'MenuText': QT_TRANSLATE_NOOP("Arch_SplitMesh","Split Mesh"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_SplitMesh","Splits selected meshes into independent components")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        if FreeCADGui.Selection.getSelection():
            sel = FreeCADGui.Selection.getSelection()
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Split Mesh"))
            for obj in sel:
                n = obj.Name
                nobjs = Arch.splitMesh(obj)
                if len(nobjs) > 1:
                    g = FreeCAD.ActiveDocument.addObject("App::DocumentObjectGroup",n)
                    for o in nobjs:
                        g.addObject(o)
            FreeCAD.ActiveDocument.commitTransaction()
            FreeCAD.ActiveDocument.recompute()


class Arch_MeshToShape:
    "the Arch MeshToShape command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_MeshToShape',
                'MenuText': QT_TRANSLATE_NOOP("Arch_MeshToShape","Mesh to Shape"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_MeshToShape","Turns selected meshes into Part Shape objects")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        from draftutils import params
        if FreeCADGui.Selection.getSelection():
            f = FreeCADGui.Selection.getSelection()[0]
            g = None
            if f.isDerivedFrom("App::DocumentObjectGroup"):
                g = f
                FreeCADGui.Selection.clearSelection()
                for o in f.OutList:
                    FreeCADGui.Selection.addSelection(o)
            else:
                if f.InList:
                    if f.InList[0].isDerivedFrom("App::DocumentObjectGroup"):
                        g = f.InList[0]
            fast = params.get_param_arch("ConversionFast")
            tol = params.get_param_arch("ConversionTolerance")
            flat = params.get_param_arch("ConversionFlat")
            cut = params.get_param_arch("ConversionCut")
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Mesh to Shape"))
            for obj in FreeCADGui.Selection.getSelection():
                newobj = Arch.meshToShape(obj,True,fast,tol,flat,cut)
                if g and newobj:
                    g.addObject(newobj)
            FreeCAD.ActiveDocument.commitTransaction()

class Arch_SelectNonSolidMeshes:
    "the Arch SelectNonSolidMeshes command definition"

    def GetResources(self):
        return {'Pixmap': 'Arch_SelectNonManifold.svg',
                'MenuText': QT_TRANSLATE_NOOP("Arch_SelectNonSolidMeshes","Select non-manifold meshes"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_SelectNonSolidMeshes","Selects all non-manifold meshes from the document or from the selected groups")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        msel = []
        if FreeCADGui.Selection.getSelection():
            for o in FreeCADGui.Selection.getSelection():
                if o.isDerivedFrom("App::DocumentObjectGroup"):
                    msel.extend(o.OutList)
        if not msel:
            msel = FreeCAD.ActiveDocument.Objects
        sel = []
        for o in msel:
            if o.isDerivedFrom("Mesh::Feature"):
                if (not o.Mesh.isSolid()) or o.Mesh.hasNonManifolds():
                    sel.append(o)
        if sel:
            FreeCADGui.Selection.clearSelection()
            for o in sel:
                FreeCADGui.Selection.addSelection(o)


class Arch_RemoveShape:
    "the Arch RemoveShape command definition"
    def GetResources(self):
        return {'Pixmap'  : 'Arch_RemoveShape',
                'MenuText': QT_TRANSLATE_NOOP("Arch_RemoveShape","Remove Shape from BIM"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_RemoveShape","Removes cubic shapes from BIM components")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        sel = FreeCADGui.Selection.getSelection()
        Arch.removeShape(sel)


class Arch_CloseHoles:
    "the Arch CloseHoles command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_CloseHoles',
                'MenuText': QT_TRANSLATE_NOOP("Arch_CloseHoles","Close holes"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_CloseHoles","Closes holes in open shapes, turning them solids")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        for o in FreeCADGui.Selection.getSelection():
            s = Arch.closeHole(o.Shape)
            if s:
                o.Shape = s


class Arch_Check:
    "the Arch Check command definition"
    def GetResources(self):
        return {'Pixmap'  : 'Arch_Check',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Check","Check"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Check","Checks the selected objects for problems")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        result = Arch.check(FreeCADGui.Selection.getSelection())
        if not result:
            FreeCAD.Console.PrintMessage(str(translate("Arch","All good! No problems found")))
        else:
            FreeCADGui.Selection.clearSelection()
            for i in result:
                FreeCAD.Console.PrintWarning("Object "+i[0].Name+" ("+i[0].Label+") "+i[1])
                FreeCADGui.Selection.addSelection(i[0])


class Arch_Survey:
    "the Arch Survey command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_Survey',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Survey","Survey"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Survey","Starts survey")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        FreeCADGui.addModule("Arch")
        FreeCADGui.doCommandGui("Arch.survey()")


class Arch_ToggleIfcBrepFlag:
    "the Toggle IFC B-rep flag command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_ToggleIfcBrepFlag',
                'MenuText': QT_TRANSLATE_NOOP("Arch_ToggleIfcBrepFlag","Toggle IFC B-rep flag"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_ToggleIfcBrepFlag","Force an object to be exported as B-rep or not")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Arch
        for o in FreeCADGui.Selection.getSelection():
            Arch.toggleIfcBrepFlag(o)


class Arch_Component:
    "the Arch Component command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_Component',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Component","Component"),
                'Accel': "C, M",
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Component","Creates an undefined architectural component")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        sel = FreeCADGui.Selection.getSelection()
        if sel:
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Create Component"))
            FreeCADGui.addModule("Arch")
            FreeCADGui.addModule("Draft")
            FreeCADGui.Control.closeDialog()
            for o in sel:
                FreeCADGui.doCommand("obj = Arch.makeComponent(FreeCAD.ActiveDocument."+o.Name+")")
                FreeCADGui.doCommand("Draft.autogroup(obj)")
            FreeCAD.ActiveDocument.commitTransaction()
            FreeCAD.ActiveDocument.recompute()


class Arch_CloneComponent:
    "the Arch Clone Component command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_Component_Clone',
                'MenuText': QT_TRANSLATE_NOOP("Arch_CloneComponent","Clone component"),
                'Accel': "C, C",
                'ToolTip': QT_TRANSLATE_NOOP("Arch_CloneComponent","Clones an object as an undefined architectural component")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        sel = FreeCADGui.Selection.getSelection()
        if sel:
            FreeCAD.ActiveDocument.openTransaction(translate("Arch","Create Component"))
            FreeCADGui.addModule("Arch")
            FreeCADGui.addModule("Draft")
            FreeCADGui.Control.closeDialog()
            for o in sel:
                FreeCADGui.doCommand("obj = Arch.cloneComponent(FreeCAD.ActiveDocument."+o.Name+")")
                FreeCADGui.doCommand("Draft.autogroup(obj)")
            FreeCAD.ActiveDocument.commitTransaction()
            FreeCAD.ActiveDocument.recompute()


class Arch_IfcSpreadsheet:
    "the Arch Schedule command definition"

    def GetResources(self):
        return {'Pixmap': 'Arch_Schedule',
                'MenuText': QT_TRANSLATE_NOOP("Arch_IfcSpreadsheet","Create IFC spreadsheet..."),
                'Accel': "I, P",
                'ToolTip': QT_TRANSLATE_NOOP("Arch_IfcSpreadsheet","Creates a spreadsheet to store IFC properties of an object.")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        sel = FreeCADGui.Selection.getSelection()
        FreeCAD.ActiveDocument.openTransaction(translate("Arch","Create IFC properties spreadsheet"))
        FreeCADGui.addModule("Arch")
        FreeCADGui.Control.closeDialog()
        if sel:
            for o in sel:
                FreeCADGui.doCommand("Arch.makeIfcSpreadsheet(FreeCAD.ActiveDocument."+o.Name+")")
        else :
            FreeCADGui.doCommand("Arch.makeIfcSpreadsheet()")
        FreeCAD.ActiveDocument.commitTransaction()
        FreeCAD.ActiveDocument.recompute()


class Arch_ToggleSubs:
    "the ToggleSubs command definition"

    def GetResources(self):
        return {'Pixmap'  : 'Arch_ToggleSubs',
                'Accel'   : 'Ctrl+Space',
                'MenuText': QT_TRANSLATE_NOOP("Arch_ToggleSubs","Toggle subcomponents"),
                'ToolTip' : QT_TRANSLATE_NOOP("Arch_ToggleSubs","Shows or hides the subcomponents of this object")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        import Draft
        mode = None
        for obj in FreeCADGui.Selection.getSelection():
            if hasattr(obj, "Subtractions"):
                for sub in obj.Subtractions:
                    if not (Draft.getType(sub) in ["Window","Roof"]):
                        if mode is None:
                            # take the first sub as base
                            mode = sub.ViewObject.isVisible()
                        if mode:
                            sub.ViewObject.hide()
                        else:
                            sub.ViewObject.show()


class Arch_MergeWalls:
    """The command definition for the Arch workbench's gui tool, Arch MergeWalls.

    A tool for merging walls.

    Join two or more walls by using the ArchWall.joinWalls() function.

    Find documentation on the end user usage of Arch Wall here:
    https://wiki.freecad.org/Arch_MergeWalls
    """

    def GetResources(self):
        """Returns a dictionary with the visual aspects of the Arch MergeWalls tool."""

        return {'Pixmap'  : 'Arch_MergeWalls',
                'MenuText': QT_TRANSLATE_NOOP("Arch_MergeWalls","Merge Walls"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_MergeWalls","Merges the selected walls, if possible")}

    def IsActive(self):
        """Determines whether or not the Arch MergeWalls tool is active.

        Inactive commands are indicated by a greyed-out icon in the menus and
        toolbars.
        """

        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v and bool(FreeCADGui.Selection.getSelection())

    def Activated(self):
        """Executed when Arch MergeWalls is called.

        Call ArchWall.joinWalls() on walls selected by the user, with the
        delete option enabled. If the user has selected a single wall, check to
        see if the wall has any Additions that are walls. If so, merges these
        additions to the wall, deleting the additions.
        """

        import Draft
        walls = FreeCADGui.Selection.getSelection()
        if len(walls) == 1:
            if Draft.getType(walls[0]) == "Wall":
                ostr = "FreeCAD.ActiveDocument."+ walls[0].Name
                ok = False
                for o in walls[0].Additions:
                    if Draft.getType(o) == "Wall":
                        ostr += ",FreeCAD.ActiveDocument." + o.Name
                        ok = True
                if ok:
                    FreeCAD.ActiveDocument.openTransaction(translate("Arch","Merge Walls"))
                    FreeCADGui.addModule("Arch")
                    FreeCADGui.doCommand("Arch.joinWalls(["+ostr+"],delete=True)")
                    FreeCAD.ActiveDocument.commitTransaction()
                    return
                else:
                    FreeCAD.Console.PrintWarning(translate("Arch","The selected wall contains no subwall to merge"))
                    return
            else:
                FreeCAD.Console.PrintWarning(translate("Arch","Please select only wall objects"))
                return
        for w in walls:
            if Draft.getType(w) != "Wall":
                FreeCAD.Console.PrintMessage(translate("Arch","Please select only wall objects"))
                return
        FreeCAD.ActiveDocument.openTransaction(translate("Arch","Merge Walls"))
        FreeCADGui.addModule("Arch")
        FreeCADGui.doCommand("Arch.joinWalls(FreeCADGui.Selection.getSelection(),delete=True)")
        FreeCAD.ActiveDocument.commitTransaction()


FreeCADGui.addCommand('Arch_Add',Arch_Add())
FreeCADGui.addCommand('Arch_Remove',Arch_Remove())
FreeCADGui.addCommand('Arch_SplitMesh',Arch_SplitMesh())
FreeCADGui.addCommand('Arch_MeshToShape',Arch_MeshToShape())
FreeCADGui.addCommand('Arch_SelectNonSolidMeshes',Arch_SelectNonSolidMeshes())
FreeCADGui.addCommand('Arch_RemoveShape',Arch_RemoveShape())
FreeCADGui.addCommand('Arch_CloseHoles',Arch_CloseHoles())
FreeCADGui.addCommand('Arch_Check',Arch_Check())
FreeCADGui.addCommand('Arch_Survey',Arch_Survey())
FreeCADGui.addCommand('Arch_ToggleIfcBrepFlag',Arch_ToggleIfcBrepFlag())
FreeCADGui.addCommand('Arch_Component',Arch_Component())
FreeCADGui.addCommand('Arch_CloneComponent',Arch_CloneComponent())
FreeCADGui.addCommand('Arch_IfcSpreadsheet',Arch_IfcSpreadsheet())
FreeCADGui.addCommand('Arch_ToggleSubs',Arch_ToggleSubs())
FreeCADGui.addCommand('Arch_MergeWalls',Arch_MergeWalls())



# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2024 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""BIM fence command"""


import os
import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate
PARAMS = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/BIM")


class Arch_Fence:
    "the Arch Fence command definition"

    def GetResources(self):
        return {'Pixmap': 'Arch_Fence',
                'MenuText': QT_TRANSLATE_NOOP("Arch_Fence", "Fence"),
                'ToolTip': QT_TRANSLATE_NOOP("Arch_Fence", "Creates a fence object from a selected section, post and path")}

    def IsActive(self):
        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):
        if len(FreeCADGui.Selection.getSelection()) != 3:
            FreeCAD.Console.PrintError(translate('Arch Fence selection','Select a section, post and path in exactly this order to build a fence.')+"\n")
            return
        doc = FreeCAD.ActiveDocument
        doc.openTransaction(translate("Arch","Create Fence"))
        FreeCADGui.addModule("Arch")
        FreeCADGui.doCommand("section = FreeCADGui.Selection.getSelection()[0]")
        FreeCADGui.doCommand("post = FreeCADGui.Selection.getSelection()[1]")
        FreeCADGui.doCommand("path = FreeCADGui.Selection.getSelection()[2]")
        FreeCADGui.doCommand("Arch.makeFence(section, post, path)")
        doc.commitTransaction()
        doc.recompute()

FreeCADGui.addCommand('Arch_Fence', Arch_Fence())



# -*- coding: utf8 -*-

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2017 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

"""This module contains BIM wrappers for commands from other workbenches"""


import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP


class BIM_Text:
    def GetResources(self):
        return {
            "Pixmap": "Draft_Text",
            "MenuText": QT_TRANSLATE_NOOP("BIM_Text", "Text"),
            "ToolTip": QT_TRANSLATE_NOOP(
                "BIM_Text", "Create a text in the current 3D view or TechDraw page"
            ),
            "Accel": "T, E",
        }

    def IsActive(self):
        act_win = FreeCADGui.getMainWindow().getActiveWindow()
        return hasattr(act_win, "getSceneGraph") or hasattr(act_win, "getPage")

    def Activated(self):
        import draftutils.utils as utils

        self.view = FreeCADGui.ActiveDocument.ActiveView
        if hasattr(self.view, "getPage") and self.view.getPage():
            self.text = ""
            FreeCADGui.draftToolBar.sourceCmd = self
            FreeCADGui.draftToolBar.taskUi()
            FreeCADGui.draftToolBar.textUi()
        else:
            FreeCADGui.runCommand("Draft_Text")

    def createObject(self):
        import TechDraw

        if self.text:
            page = self.view.getPage()
            pagescale = page.Scale
            if not pagescale:
                pagescale = 1
            anno = FreeCAD.ActiveDocument.addObject(
                "TechDraw::DrawViewAnnotation", "Annotation"
            )
            anno.Text = self.text
            page.addView(anno)
            param = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
            anno.TextSize = param.GetFloat("textheight", 10) * pagescale
            anno.Font = param.GetString("textfont", "Sans")
            c = param.GetUnsigned("DefaultTextColor", 255)
            r = ((c >> 24) & 0xFF) / 255.0
            g = ((c >> 16) & 0xFF) / 255.0
            b = ((c >> 8) & 0xFF) / 255.0
            anno.TextColor = (r, g, b)
            self.finish()

    def finish(self, arg=False):
        FreeCADGui.draftToolBar.sourceCmd = None
        FreeCADGui.draftToolBar.offUi()


FreeCADGui.addCommand("BIM_Text", BIM_Text())

